//| ScalpingMa.mq5 |
//| Copyright 2009-2020, MetaQuotes Software Corp. |
//| http://www.mql5.com |
//+------------------------------------------------------------------+

#property copyright "Copyright ?2005, MetaQuotes Software Corp."
#property link "http://www.metaquotes.net/"
#include <MovingAverages.mqh>

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_plots 1
#property indicator_type1 DRAW_LINE
#property indicator_color1 clrRed
#property indicator_style1 STYLE_SOLID
#property indicator_width1 4

extern bool NullBarRecount = true;

//--- input parametrs
input int mPeriod=6; // Period
input ENUM_TIMEFRAMES timeframe = 0; //timeframe
//--- global variables
int ExtBandsPeriod;
int ExtBandsShift = 0;
double ExtBandsDeviations;
int ExtPlotBegin=0;
//--- indicator buffer
double ExtMLBuffer[];


//+------------------------------------------------------------------+
//| Custom indicator initialization function |
//+------------------------------------------------------------------+
int OnInit()
  {
////--- indicator buffers mapping
   SetIndexBuffer(0, ExtMLBuffer, INDICATOR_DATA);
// SetIndexBuffer(1,InputBuffer1,INDICATOR_CALCULATIONS);
   ArraySetAsSeries(ExtMLBuffer, true);
//--- indicator buffers mapping
//--- set accuracy
   IndicatorSetInteger(INDICATOR_DIGITS, _Digits+1);
//--- set first bar from what index will be drawn
   PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, 0);
//--- line shifts when drawing
   PlotIndexSetInteger(0, PLOT_SHIFT, ExtBandsShift);
   IndicatorSetString(INDICATOR_SHORTNAME, "BB_scalping"+"("+string(mPeriod)+")");
//--- set drawing line empty value
   PlotIndexSetDouble(0, PLOT_EMPTY_VALUE, 0.0);
   return(0);
  }

//+------------------------------------------------------------------+
//| Bollinger Bands |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const int begin,
                const double &price[])
  {
   ArraySetAsSeries(price, true);
   if(rates_total<2*mPeriod-1+begin)
      return(0);
//--- first calculation or number of bars was changed

//--- indexes draw begin settings, when we've recieved previous begin
   if(ExtPlotBegin!=mPeriod+begin)
     {
      ExtPlotBegin=mPeriod+begin;
      PlotIndexSetInteger(0, PLOT_DRAW_BEGIN, ExtPlotBegin);
     }
//--- starting calculation
   int pos = rates_total - ( ( prev_calculated < 1 ) ? 1 : prev_calculated );
//--- main cycle
   for(int i=pos; i>=0 && !IsStopped(); i--)
     {
      //--- indexes draw begin settings, when we've recieved previous begin


      ExtMLBuffer[i] = Get_SCASeries(rates_total, prev_calculated, Symbol(), NullBarRecount, price, timeframe, mPeriod, ExtMLBuffer);

     }
//--- OnCalculate done. Return new prev_calculated.
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool Get_SCASeries(const int ratesTotal, const int prevCalculated, string symbol, bool NullBarRcount, const double& Price[],
                   ENUM_TIMEFRAMES time_frame, int m_Period, double& InputBuffer1[])

  {

//---- Checking whether the bars number is enough for further calculation
   if(ratesTotal <2*m_Period +4)
      return(false);

//--- starting calculation
   int pos;
   if(prevCalculated>1)
      pos=prevCalculated-1;
   else
      pos=0;

   int LastCountBar = 0;

   if(!NullBarRcount)
      LastCountBar = 1;

   int limit = 0, i = 0;

/////////////////////////////////////////////////
   if(prevCalculated < 0)
      return(false);
   if(prevCalculated == 0)
     {

      limit=ratesTotal- (2*m_Period)-3;
     }
//---- the last calculated bar must be recalculated
   if(prevCalculated > 0)

      limit = ratesTotal- prevCalculated-(2*m_Period)-2;

   for(i=limit; i>=pos&&!IsStopped(); i--)

     {
      InputBuffer1[i] = ExponentialMA(i, m_Period, InputBuffer1[i-1], Price);

      int count_bar_1;
      int count_bar_2;
      int smoothed_b;
      int smoothed_f;
      double cum_value;
      int routine_num;
      for(routine_num = i + m_Period; routine_num >= i; routine_num--)
        {

         cum_value = 0.0;
         count_bar_1 = 0;
         count_bar_2 = 0;
         smoothed_b = routine_num + m_Period;
         smoothed_f = routine_num - m_Period;
         if(smoothed_f<i)
            smoothed_f = i;

         while(smoothed_b >= routine_num)
           {
            count_bar_1++;
            cum_value += count_bar_1*Price[smoothed_b];
            count_bar_2 += count_bar_1;
            smoothed_b--;
           }

         while(smoothed_b >= smoothed_f)
           {
            count_bar_1--;
            cum_value += count_bar_1*Price[smoothed_b];
            count_bar_2 += count_bar_1;
            smoothed_b--;

           }

         InputBuffer1[routine_num] = cum_value / count_bar_2;

        }
     }
   return(true);
  }
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
